@@ -1,7 +1,7 @@
 #
 # closepairs.R
 #
-#   $Revision: 1.47 $   $Date: 2021/01/07 01:15:08 $
+#   $Revision: 1.54 $   $Date: 2022/06/15 01:35:50 $
 #
 #  simply extract the r-close pairs from a dataset
 # 
@@ -65,13 +65,22 @@ closepairs.ppp <- function(X, rmax, twice=TRUE,
   if(!is.null(nsize)) {
     splat("Using nsize =", nsize)
   } else {
-    #' normal usage 
-    nsize <- ceiling(2 * pi * (npts^2) * (rmax^2)/area(Window(X)))
-    nsize <- max(1024, nsize)
-    if(nsize > .Machine$integer.max) {
-     warning("Estimated number of close pairs exceeds maximum possible integer",
-              call.=FALSE)
-      nsize <- .Machine$integer.max
+    #' normal usage
+    #' calculate a conservative estimate of the number of pairs
+    npairs <- as.double(npts)^2
+    if(npairs <= 1024) {
+      nsize <- 1024
+    } else {
+      catchfraction <- pi * (rmax^2)/area(Frame(X))
+      nsize <- ceiling(2 * catchfraction * npairs)
+      nsize <- min(nsize, npairs)
+      nsize <- max(1024, nsize)
+      if(nsize > .Machine$integer.max) {
+        warning(
+          "Estimated number of close pairs exceeds maximum possible integer",
+          call.=FALSE)
+        nsize <- .Machine$integer.max
+      }
     }
   }
   ## Now extract pairs
@@ -236,7 +245,7 @@ closepairs.ppp <- function(X, rmax, twice=TRUE,
     }
 
     got.twice <- TRUE
-    nsize <- nsize * 2
+    nsize <- as.integer(min(as.double(nsize) * 2, as.double(.Machine$integer.max)))
     z <-
       .C(SG_Fclosepairs,
          nxy=as.integer(npts),
@@ -410,7 +419,7 @@ closepairs.ppp <- function(X, rmax, twice=TRUE,
              xi <- c(xi, xx)
              yi <- c(yi, yy)
              xj <- c(xj, xx)
-             yi <- c(yi, yy)
+             yj <- c(yj, yy)
              dx <- c(dx, zeroes)
              dy <- c(dy, zeroes)
              d  <- c(d, zeroes)
@@ -444,7 +453,10 @@ crosspairs <- function(X, Y, rmax, ...) {
   UseMethod("crosspairs")
 }
 
-crosspairs.ppp <- function(X, Y, rmax, what=c("all", "indices", "ijd"), ...) {
+crosspairs.ppp <- function(X, Y, rmax,
+                           what=c("all", "indices", "ijd"),
+                           periodic=FALSE, ...,
+                           iX=NULL, iY=NULL) {
   verifyclass(X, "ppp")
   verifyclass(Y, "ppp")
   what <- match.arg(what)
@@ -473,21 +485,96 @@ crosspairs.ppp <- function(X, Y, rmax, what=c("all", "indices", "ijd"), ...) {
   nX <- npoints(X)
   nY <- npoints(Y)
   if(nX == 0 || nY == 0) return(null.answer)
-  # order patterns by increasing x coordinate
+  nXY <- as.double(nX) * as.double(nY)
+  if(periodic) {
+    ## ............... Periodic distance ..................
+    if(!is.rectangle(Window(Y)))
+      warning("Periodic edge correction applied in non-rectangular window",
+              call.=FALSE)
+    ## Overestimate the number of pairs
+    if(nXY <= 1024) {
+      nsize <- 1024
+    } else {
+      catchfraction <- pi * (rmax^2)/area(Frame(Y))
+      nsize <- ceiling(2 * catchfraction * nXY)
+      nsize <- min(nsize, nXY)
+      nsize <- max(1024, nsize)
+      if(nsize > .Machine$integer.max) {
+        warning(
+          "Estimated number of close pairs exceeds maximum possible integer",
+          call.=FALSE)
+        nsize <- .Machine$integer.max
+      }
+    }
+    # .Call
+    Xx <- X$x
+    Xy <- X$y
+    Yx <- Y$x
+    Yy <- Y$y
+    per <- sidelengths(Frame(Y))
+    r <- rmax
+    ng <- nsize
+    storage.mode(Xx) <- storage.mode(Xy) <- "double"
+    storage.mode(Yx) <- storage.mode(Yy) <- "double"
+    storage.mode(per) <- storage.mode(r) <- "double"
+    storage.mode(ng) <- "integer"
+    z <- .Call(SG_crossPpair,
+               xxA=Xx, yyA=Xy,
+               xxB=Yx, yyB=Yy,
+               pp=per, rr=r, nguess=ng,
+               PACKAGE="spatstat.geom")
+    if(length(z) != 3)
+      stop("Internal error: incorrect format returned from crossPpair")
+    i  <- z[[1L]]  # NB no increment required
+    j  <- z[[2L]]
+    d  <- z[[3L]]
+    answer <- switch(what,
+                     indices = list(i=i, j=j),
+                     ijd =     list(i=i, j=j, d=d),
+                     all = {
+                       xi <- Xx[i]
+                       yi <- Xy[i]
+                       xj <- Yx[j]
+                       yj <- Yy[j]
+                       dx <- xj-xi
+                       dy <- yj-yi
+                       list(i=i,
+                            j=j,
+                            xi=xi, 
+                            yi=yi,
+                            xj=xj,
+                            yj=yj,
+                            dx=dx,
+                            dy=dy,
+                            d=d)
+                     })
+    if(!is.null(iX) && !is.null(iY))
+      answer <- remove.identical.pairs(answer, iX, iY)
+    return(answer)
+  }
+  ## .......... Euclidean distance .......................
+  ## order patterns by increasing x coordinate
   ooX <- fave.order(X$x)
   Xsort <- X[ooX]
   ooY <- fave.order(Y$x)
   Ysort <- Y[ooY]
   if(spatstat.options("crosspairs.newcode")) {
-    # ------------------- use new faster code ---------------------
-    # First (over)estimate the number of pairs
-    nsize <- ceiling(2 * pi * (rmax^2) * nX * nY/area(Window(Y)))
-    nsize <- max(1024, nsize)
-    if(nsize > .Machine$integer.max) {
-      warning(
-        "Estimated number of close pairs exceeds maximum possible integer",
-        call.=FALSE)
-      nsize <- .Machine$integer.max
+    ## ------------------- use new faster code ---------------------
+    ## First (over)estimate the number of pairs
+    nXY <- as.double(nX) * as.double(nY)
+    if(nXY <= 1024) {
+      nsize <- 1024
+    } else {
+      catchfraction <- pi * (rmax^2)/area(Frame(Y))
+      nsize <- ceiling(2 * catchfraction * nXY)
+      nsize <- min(nsize, nXY)
+      nsize <- max(1024L, nsize)
+      if(nsize > .Machine$integer.max) {
+        warning(
+          "Estimated number of close pairs exceeds maximum possible integer",
+          call.=FALSE)
+        nsize <- .Machine$integer.max
+      }
     }
     # .Call
     Xx <- Xsort$x
@@ -626,6 +713,8 @@ crosspairs.ppp <- function(X, Y, rmax, what=c("all", "indices", "ijd"), ...) {
          ijd = {
            answer <- list(i=i, j=j, d=d)
          })
+  if(!is.null(iX) && !is.null(iY))
+    answer <- remove.identical.pairs(answer, iX, iY)
   return(answer)
 }
 
@@ -643,18 +732,26 @@ closethresh <- function(X, R, S, twice=TRUE, ...) {
     twice <- ordered
   }
   npts <- npoints(X)
-   if(npts == 0)
+   if(npts <= 1)
      return(list(i=integer(0), j=integer(0), t=logical(0)))
   # sort points by increasing x coordinate
   oo <- fave.order(X$x)
   Xsort <- X[oo]
-  # First make an OVERESTIMATE of the number of pairs
-  nsize <- ceiling(4 * pi * (npts^2) * (R^2)/area(Window(X)))
-  nsize <- max(1024, nsize)
-  if(nsize > .Machine$integer.max) {
-    warning("Estimated number of close pairs exceeds maximum possible integer",
-            call.=FALSE)
-    nsize <- .Machine$integer.max
+  ## First make an OVERESTIMATE of the number of pairs
+  npairs <- as.double(npts) * (as.double(npts) - 1)
+  if(npairs <= 1024) {
+    nsize <- 1024
+  } else {
+    catchfraction <- pi * (R^2)/area(Frame(X))
+    nsize <- ceiling(4 * catchfraction * npairs)
+    nsize <- min(nsize, npairs)
+    nsize <- max(1024, nsize)
+    if(nsize > .Machine$integer.max) {
+      warning(
+        "Estimated number of close pairs exceeds maximum possible integer",
+        call.=FALSE)
+      nsize <- .Machine$integer.max
+    }
   }
   # Now extract pairs
   x <- Xsort$x
@@ -764,3 +861,11 @@ tweak.closepairs <- function(cl, rmax, i, deltax, deltay, deltaz) {
   return(cl)
 }
 
+remove.identical.pairs <- function(cl, imap, jmap) {
+  ## 'cl' is the result of crosspairs
+  ## 'imap', 'jmap' map the 'i' and 'j' indices to a common sequence
+  distinct <- (imap[cl$i] != jmap[cl$j])
+  if(!all(distinct))
+    cl <- lapply(cl, "[", i=distinct)
+  return(cl)
+}
