@@ -3,7 +3,7 @@
 #
 # support for colour maps and other lookup tables
 #
-# $Revision: 1.47 $ $Date: 2020/07/03 03:21:28 $
+# $Revision: 1.63 $ $Date: 2024/12/02 01:41:18 $
 #
 
 colourmap <- function(col, ..., range=NULL, breaks=NULL, inputs=NULL, gamma=1) {
@@ -219,12 +219,48 @@ plot.colourmap <- local({
     if(dw == 0) 1 else (dw * (if(separate) (n + (n-1)*gap) else 10))
   }
 
+  sideCode <- function(side) {
+    if(is.numeric(side)) {
+      stopifnot(side %in% 1:4)
+      sidecode <- side
+    } else if(is.character(side)) {
+      nama <- c("bottom", "left", "top", "right")
+      side <- match.arg(side, nama)
+      sidecode <- match(side, nama)
+    } else stop("Unrecognised format for 'side'")
+    return(sidecode)
+  }
+
+  Ticks <- function(usr, log=FALSE, nint=NULL, ..., clip=TRUE) {
+    #' modification of grDevices::axisTicks
+    #'      constrains ticks to be inside the specified range 'usr' if clip=TRUE
+    #'      accepts nint=NULL as if it were missing
+    z <- if(is.null(nint)) axisTicks(usr=usr, log=log, ...) else
+         axisTicks(usr=usr, log=log, nint=nint, ...) 
+    if(clip) {
+      zlimits <- if(log) 10^usr else usr
+      z <- z[inside.range(z, zlimits)]
+    }
+    return(z)
+  }
+
+    
   plot.colourmap <- function(x, ..., main,
-                             xlim=NULL, ylim=NULL, vertical=FALSE, axis=TRUE,
+                             xlim=NULL, ylim=NULL,
+                             vertical=FALSE,
+                             axis=TRUE,
+                             side = if(vertical) "right" else "bottom",
                              labelmap=NULL, gap=0.25, add=FALSE,
-                             increasing=NULL) {
+                             increasing=NULL, nticks=5, box=NULL) {
     if(missing(main))
       main <- short.deparse(substitute(x))
+    if(missing(vertical) && !missing(side)) 
+      vertical <- (sideCode(side) %in% c(2, 4))
+    
+    dotargs <- list(...)
+    if(inherits(dotargs$col, "colourmap"))
+      dotargs <- dotargs[names(dotargs) != "col"]
+    
     stuff <- attr(x, "stuff")
     col <- stuff$outputs
     n   <- stuff$n
@@ -300,7 +336,7 @@ plot.colourmap <- local({
                                             type="n", main=main,
                                             axes=FALSE, xlab="", ylab="",
                                             asp=1.0),
-                                       list(...)))
+                                       dotargs))
 
     if(separate) {
       # ................ plot separate blocks of colour .................
@@ -317,11 +353,11 @@ plot.colourmap <- local({
           do.call.matched(image.default,
                           resolve.defaults(list(x=ensurenumeric(x),
                                                 y=ensurenumeric(y),
-                                                z=z, add=TRUE),
-                                       list(...),
-                                       list(col=col[i])),
-                      extrargs=imageparams)
-                          
+                                                z=z,
+                                                add=TRUE,
+                                                col=col[i]),
+                                           dotargs),
+                          extrargs=imageparams)
         }
       } else {
         # vertical arrangement of blocks
@@ -334,11 +370,11 @@ plot.colourmap <- local({
           do.call.matched(image.default,
                           resolve.defaults(list(x=ensurenumeric(x),
                                                 y=ensurenumeric(y),
-                                                z=z, add=TRUE),
-                                       list(...),
-                                       list(col=col[i])),
-                      extrargs=imageparams)
-                          
+                                                z=z,
+                                                add=TRUE,
+                                                col=col[i]),
+                                           dotargs),
+                          extrargs=imageparams)
         }
       }
     } else {
@@ -372,12 +408,19 @@ plot.colourmap <- local({
       if(reverse)
         col <- rev(col)
       do.call.matched(image.default,
-                      resolve.defaults(list(x=x, y=y, z=z, add=TRUE),
-                                       list(...),
-                                       list(breaks=ensurenumeric(bks),
-                                            col=col)),
+                      resolve.defaults(list(x=x, y=y, z=z,
+                                            add=TRUE,
+                                            breaks=ensurenumeric(bks),
+                                            col=col),
+                                       dotargs),
                       extrargs=imageparams)
     }
+    #' draw box around colours?
+    #' default is TRUE unless drawing blocks of colour with gaps between.
+    if(is.null(box)) box <- !separate
+    if(!isFALSE(box))
+      rect(xlim[1], ylim[1], xlim[2], ylim[2])
+
     if(axis) {
       # ................. draw annotation ..................
       if(!vertical) {
@@ -386,26 +429,28 @@ plot.colourmap <- local({
           la <- paste(labelmap(stuff$inputs))
           at <- linmap(v, rr, xlim)
         } else {
-          la <- prettyinside(rr)
+          la <- Ticks(rr, nint=nticks)
           at <- linmap(la, rr, xlim)
           la <- labelmap(la)
         }
         if(reverse)
           at <- rev(at)
         # default axis position is below the ribbon (side=1)
-        sidecode <- resolve.1.default("side", list(...), list(side=1L))
+        sidecode <- sideCode(side)
         if(!(sidecode %in% c(1L,3L)))
-          warning(paste("side =", sidecode,
+          warning(paste("side =", 
+                        if(is.character(side)) sQuote(side) else side,
                         "is not consistent with horizontal orientation"))
         pos <- c(ylim[1L], xlim[1L], ylim[2L], xlim[2L])[sidecode]
         # don't draw axis lines if plotting separate blocks
         lwd0 <- if(separate) 0 else 1
         # draw axis
         do.call.matched(graphics::axis,
-                        resolve.defaults(list(...),
-                                         list(side = 1L, pos = pos,
-                                              at = ensurenumeric(at)),
-                                         list(labels=la, lwd=lwd0)),
+                        resolve.defaults(dotargs,
+                                         list(side = sidecode,
+                                              pos = pos,
+                                              at = ensurenumeric(at),
+                                              labels=la, lwd=lwd0)),
                         extrargs=axisparams)
       } else {
         # add vertical axis
@@ -413,16 +458,17 @@ plot.colourmap <- local({
           la <- paste(labelmap(stuff$inputs))
           at <- linmap(v, rr, ylim)
         } else {
-          la <- prettyinside(rr)
+          la <- Ticks(rr, nint=nticks)
           at <- linmap(la, rr, ylim)
           la <- labelmap(la)
         }
         if(reverse)
           at <- rev(at)
         # default axis position is to the right of ribbon (side=4)
-        sidecode <- resolve.1.default("side", list(...), list(side=4))
+        sidecode <- sideCode(side)
         if(!(sidecode %in% c(2L,4L)))
-          warning(paste("side =", sidecode,
+          warning(paste("side =",
+                        if(is.character(side)) sQuote(side) else side,
                         "is not consistent with vertical orientation"))
         pos <- c(ylim[1L], xlim[1L], ylim[2L], xlim[2L])[sidecode]
         # don't draw axis lines if plotting separate blocks
@@ -431,10 +477,11 @@ plot.colourmap <- local({
         las0 <- if(separate) 1 else 0
         # draw axis
         do.call.matched(graphics::axis,
-                        resolve.defaults(list(...),
-                                         list(side=4, pos=pos,
-                                              at=ensurenumeric(at)),
-                                         list(labels=la, lwd=lwd0, las=las0)),
+                        resolve.defaults(dotargs,
+                                         list(side=sidecode,
+                                              pos=pos,
+                                              at=ensurenumeric(at),
+                                              labels=la, lwd=lwd0, las=las0)),
                         extrargs=axisparams)
       }
     }
@@ -584,3 +631,73 @@ colouroutputs <- function(x) {
   return(x)
 }
 
+rev.colourmap <- function(x) {
+  colouroutputs(x) <- rev(colouroutputs(x))
+  return(x)
+}
+
+restrict.colourmap <- function(x, ..., range=NULL, breaks=NULL, inputs=NULL) {
+  stopifnot(inherits(x, "colourmap"))
+  given <- c(!is.null(range), !is.null(breaks), !is.null(inputs))
+  names(given) <- nama <- c("range", "breaks", "inputs")
+  ngiven <- sum(given)
+  if(ngiven == 0L)
+    return(x)
+  if(ngiven > 1L) {
+    offending <- nama[given]
+    stop(paste("The arguments",
+               commasep(sQuote(offending)),
+               "are incompatible"))
+  }
+  stuff <- attr(x, "stuff")
+  if(!is.null(inputs)) {
+    ## discrete colour map
+    if(!stuff$discrete) 
+      stop("Cannot update 'inputs'; the existing colour map is not discrete",
+           call.=FALSE)
+    oldinputs <- stuff$inputs
+    oldoutputs <- stuff$outputs
+    m <- match(inputs, oldinputs)
+    if(any(is.na(m)))
+      stop("New inputs are not a subset of the old inputs", call.=FALSE)
+    result <- colourmap(oldoutputs[m], inputs=inputs)
+  } else if(!is.null(range)) {
+    ## colour map for continuous domain
+    ## range specified
+    if(stuff$discrete) 
+      stop("Cannot update 'range'; the existing colour map is discrete",
+           call.=FALSE)
+    check.range(range)
+    oldbreaks <- stuff$breaks
+    if(!all(inside.range(range, range(oldbreaks))))
+      stop("new range of values is not a subset of current range")
+    ## restrict existing breaks to new range
+    newbreaks <- pmax(range[1], pmin(range[2], oldbreaks))
+    newbreaks <- unique(newbreaks)
+    ## evaluate current colour at midpoint of each new interval
+    newmid <- newbreaks[-length(newbreaks)] + diff(newbreaks)/2
+    newout <- x(newmid)
+    result <- colourmap(newout, breaks=newbreaks)
+  } else {
+    ## colour map for continuous domain
+    ## breaks specified
+    if(stuff$discrete) 
+      stop("Cannot update 'breaks'; the existing colour map is discrete",
+           call.=FALSE)
+    oldbreaks <- stuff$breaks
+    if(!all(inside.range(breaks, range(oldbreaks))))
+      stop("new range of 'breaks' is not a subset of current range of 'breaks'",
+           call.=FALSE)
+    newmid <- breaks[-length(breaks)] + diff(breaks)/2
+    newout <- x(newmid)
+    result <- colourmap(newout, breaks=breaks)
+  }
+  return(result)
+}
+
+as.colourmap <- function(x, ...) {
+  UseMethod("as.colourmap")
+}
+
+as.colourmap.colourmap <- function(x, ...) { x }
+
