@@ -1,7 +1,7 @@
 #
 #   plot.im.R
 #
-#  $Revision: 1.155 $   $Date: 2022/05/23 02:33:06 $
+#  $Revision: 1.161 $   $Date: 2024/12/03 01:19:40 $
 #
 #  Plotting code for pixel images
 #
@@ -31,7 +31,7 @@ plot.im <- local({
                       resolve.defaults(list(x=quote(W), type="n"), aarg), 
                       extrargs=graphicsPars("owin"))
     }
-    if(workaround && identical(aarg$useRaster, TRUE)) {
+    if(workaround && isTRUE(aarg$useRaster)) {
       #' workaround for bug 16035
       #' detect reversed coordinates
       usr <- par('usr')
@@ -47,6 +47,8 @@ plot.im <- local({
       }
     }
     extrargs <- setdiff(extrargs, c("claim.title.space", "box"))
+    if(!is.na(k <- match("adj.main", names(aarg))))
+      names(aarg)[k] <- "adj"
     z <- do.call.matched(image.default,
                          append(imagedata, aarg),
                          extrargs=extrargs)
@@ -222,7 +224,7 @@ plot.im <- local({
       zlimits <- if(log) 10^usr else usr
       z <- z[inside.range(z, zlimits)]
     }
-    return(z)
+    return(unique(z))
   }
 
   numericalRange <- function(x, zlim=NULL) {
@@ -243,9 +245,11 @@ plot.im <- local({
   PlotIm <- function(x, ...,
                      main, 
                      add=FALSE, clipwin=NULL,
-                     col=NULL, valuesAreColours=NULL, log=FALSE,
+                     col=NULL, reverse.col=FALSE,
+                     valuesAreColours=NULL, log=FALSE,
                      ncolours=256, gamma=1, 
                      ribbon=show.all, show.all=!add,
+                     drop.ribbon=FALSE,
                      ribside=c("right", "left", "bottom", "top"),
                      ribsep=0.15, ribwid=0.05, ribn=1024,
                      ribscale=1, ribargs=list(), riblab=NULL, colargs=list(),
@@ -417,57 +421,55 @@ plot.im <- local({
                col <- s$outputs
              } 
              trivial <- (diff(vrange) <= zap * .Machine$double.eps)
-             #' ribbonvalues: domain of colour map (pixel values)
-             #' ribbonrange: (min, max) of pixel values in image
-             #' nominalrange: range of values shown on ribbon 
-             #' nominalmarks: values shown on ribbon at tick marks
-             #' ribbonticks: pixel values of tick marks 
+             #' ribbonvalues: a sequence of pixel values, mapped to colours
+             #' ribbonrange:  (min, max) of pixel values mapped by ribbon
+             #' nominalrange: range of (scaled) values shown on ribbon 
+             #' nominalmarks: (scaled) values shown on ribbon at tick marks
+             #' ribbonticks: pixel values corresponding to tick marks 
              #' ribbonlabels: text displayed at tick marks
-             ribbonvalues <- if(trivial) vrange[1] else
-                             seq(from=vrange[1], to=vrange[2], length.out=ribn)
-             ribbonrange <- vrange
-             nominalrange <- Log(ribscale * Exp(ribbonrange))
-             nominalmarks <- if(trivial) nominalrange[1] else
-                             (user.ticks %orifnull% Ticks(nominalrange,
+             if(trivial) {
+               ribbonvalues <- mean(vrange)
+               nominalmarks <- Log(ribscale * Exp(ribbonvalues))
+             } else {
+               ribbonvalues <- seq(from=vrange[1L], to=vrange[2L],
+                                   length.out=ribn)
+               ribbonrange <- vrange
+               nominalrange <- Log(ribscale * Exp(ribbonrange))
+               nominalmarks <- user.ticks %orifnull% Ticks(nominalrange,
                                                          log=do.log,
-                                                         nint=user.nint))
+                                                         nint=user.nint)
+             }
              ribbonticks <- Log(nominalmarks/ribscale)
              ribbonlabels <- user.ribbonlabels %orifnull% paste(nominalmarks)
            },
            integer = {
-             values <- as.vector(x$v)
-             values <- values[!is.na(values)]
-             uv <- unique(values)
-             vrange <- numericalRange(uv, zlim)
-             nvalues <- length(uv)
-             trivial <- (nvalues < 2)
-             if(!trivial){
-               nominalrange <- Log(ribscale * Exp(vrange))
-               if(!is.null(user.ticks)) {
-                 nominalmarks <- user.ticks
-               } else {
-                 nominalmarks <- Ticks(nominalrange,
-                                       log=do.log,
-                                       nint = user.nint)
-                 nominalmarks <- nominalmarks[nominalmarks %% 1 == 0]
-               }
-               ribbonticks <- Log(nominalmarks/ribscale)
-               ribbonlabels <- user.ribbonlabels %orifnull% paste(nominalmarks)
-               if(!do.log && isTRUE(all.equal(ribbonticks,
-                                              vrange[1]:vrange[2]))) {
-                 # each possible pixel value will appear in ribbon
-                 ribbonvalues <- vrange[1]:vrange[2]
-                 imagebreaks <- c(ribbonvalues - 0.5, vrange[2] + 0.5)
-                 ribbonrange <- range(imagebreaks)
-                 ribbonticks <- ribbonvalues
-                 ribbonlabels <- user.ribbonlabels %orifnull% paste(ribbonticks * ribscale)
-               } else {
-                 # not all possible values will appear in ribbon
-                 ribn <- min(ribn, diff(vrange)+1)
-                 ribbonvalues <- seq(from=vrange[1], to=vrange[2],
-                                     length.out=ribn)
-                 ribbonrange <- vrange
-               }
+             vrange <- numericalRange(x, zlim)
+             trivial <- (diff(vrange) < 1)
+             nominalrange <- Log(ribscale * Exp(vrange))
+             if(!is.null(user.ticks)) {
+               nominalmarks <- user.ticks
+             } else {
+               nominalmarks <- Ticks(nominalrange,
+                                     log=do.log,
+                                     nint = user.nint)
+               nominalmarks <- nominalmarks[nominalmarks %% 1 == 0]
+             }
+             ribbonticks <- Log(nominalmarks/ribscale)
+             ribbonlabels <- user.ribbonlabels %orifnull% paste(nominalmarks)
+             if(!do.log && isTRUE(all.equal(ribbonticks,
+                                            vrange[1]:vrange[2]))) {
+               #' each possible pixel value will appear in ribbon
+               ribbonvalues <- vrange[1]:vrange[2]
+               imagebreaks <- c(ribbonvalues - 0.5, vrange[2] + 0.5)
+               ribbonrange <- range(imagebreaks)
+               ribbonticks <- ribbonvalues
+               ribbonlabels <- user.ribbonlabels %orifnull% paste(ribbonticks * ribscale)
+             } else {
+               ## not all possible values will appear in ribbon
+               ribn <- min(ribn, diff(vrange)+1)
+               ribbonvalues <- seq(from=vrange[1], to=vrange[2],
+                                   length.out=ribn)
+               ribbonrange <- vrange
              }
              if(!is.null(colmap)) {
                # explicit colour map
@@ -479,11 +481,8 @@ plot.im <- local({
              }
            },
            logical = {
-             values <- as.integer(as.vector(x$v))
-             values <- values[!is.na(values)]
-             uv <- unique(values)
-             trivial <- (length(uv) < 2)
              vrange <- c(0,1)
+             trivial <- FALSE
              imagebreaks <- c(-0.5, 0.5, 1.5)
              ribbonvalues <- c(0,1)
              ribbonrange <- range(imagebreaks)
@@ -562,6 +561,11 @@ plot.im <- local({
       ## transform to grey scale
       colourinfo$col <- to.grey(colourinfo$col)
     }
+
+    if(isTRUE(reverse.col) && !valuesAreColours) {
+      ## reverse the colour sequence (using rev.colourmap or rev.default)
+      colourinfo$col <- rev(colourinfo$col)
+    }
     
     # colour map to be returned (invisibly)
     i.col <- colourinfo$col
@@ -592,33 +596,35 @@ plot.im <- local({
       colourinfo$breaks <- soc$breaks
 
     ##  ........ decide whether to use rasterImage .........
-    
-    ## get device capabilities
-    ##      (this will start a graphics device if none is active)
-    rasterable <- dev.capabilities()$rasterImage
-    if(is.null(rasterable)) rasterable <- "no"
-    ##
-    can.use.raster <-
-      switch(rasterable,
-             yes=TRUE,
-             no=FALSE,
-             "non-missing"=!anyNA(x$v),
-             FALSE)
-    if(is.null(useRaster)) {
-      useRaster <- can.use.raster
-    } else if(useRaster && !can.use.raster) {
+
+    if(!isFALSE(useRaster)) {
+      ## get device capabilities
+      ##      (this will start a graphics device if none is active)
+      rasterable <- safeDevCapabilities()$rasterImage
+      if(is.null(rasterable)) rasterable <- "no"
+      ##
+      can.use.raster <-
+        switch(rasterable,
+               yes=TRUE,
+               no=FALSE,
+               "non-missing"=!anyNA(x$v),
+               FALSE)
+      if(is.null(useRaster)) {
+        useRaster <- can.use.raster
+      } else if(useRaster && !can.use.raster) {
         whinge <- "useRaster=TRUE is not supported by the graphics device"
         if(rasterable == "non-missing")
           whinge <- paste(whinge, "for images with NA values")
         warning(whinge, call.=FALSE)
-    } 
+      }
+    }
 
     ## ........ catch old usage (undocumented ) ................
     contourargs <- resolve.defaults(contourargs, dotargs$args.contour)
     
     ## ........ start plotting .................
 
-    if(!identical(ribbon, TRUE) || trivial) {
+    if(!isTRUE(ribbon) || (trivial && isTRUE(drop.ribbon))) {
       ## no ribbon wanted
 
       attr(output.colmap, "bbox") <- as.rectangle(x)
@@ -650,32 +656,32 @@ plot.im <- local({
     }
     
     # determine plot region
-    bb <- owin(x$xrange, x$yrange)
+    bb <- owinInternalRect(x$xrange, x$yrange)
     Width <- diff(bb$xrange)
     Height <- diff(bb$yrange)
     Size <- max(Width, Height)
     switch(ribside,
            right={
              # ribbon to right of image
-             bb.rib <- owin(bb$xrange[2] + c(ribsep, ribsep+ribwid) * Size,
+             bb.rib <- owinInternalRect(bb$xrange[2] + c(ribsep, ribsep+ribwid) * Size,
                             bb$yrange)
              rib.iside <- 4
            },
            left={
              # ribbon to left of image
-             bb.rib <- owin(bb$xrange[1] - c(ribsep+ribwid, ribsep) * Size,
+             bb.rib <- owinInternalRect(bb$xrange[1] - c(ribsep+ribwid, ribsep) * Size,
                             bb$yrange)
              rib.iside <- 2
            },
            top={
              # ribbon above image
-             bb.rib <- owin(bb$xrange,
+             bb.rib <- owinInternalRect(bb$xrange,
                             bb$yrange[2] + c(ribsep, ribsep+ribwid) * Size)
              rib.iside <- 3
            },
            bottom={
              # ribbon below image
-             bb.rib <- owin(bb$xrange,
+             bb.rib <- owinInternalRect(bb$xrange,
                             bb$yrange[1] - c(ribsep+ribwid, ribsep) * Size)
              rib.iside <- 1
            })
@@ -790,29 +796,45 @@ plot.im <- local({
       axisargs <- list(side=ribaxis.iside, labels=ribbonlabels)
       switch(ribside,
              right={
-               scal <- diff(bb.rib$yrange)/diff(ribbonrange)
-               at <- bb.rib$yrange[1] + scal * (ribbonticks - ribbonrange[1])
+               if(trivial) {
+                 at <- mean(bb.rib$yrange)
+               } else {
+                 scal <- diff(bb.rib$yrange)/diff(ribbonrange)
+                 at <- bb.rib$yrange[1] + scal * (ribbonticks - ribbonrange[1])
+               }
                axisargs <- append(axisargs, list(at=at))
                posargs <- list(pos=bb.rib$xrange[2],
                                yaxp=c(bb.rib$yrange, length(ribbonticks)))
              },
              left={
-               scal <- diff(bb.rib$yrange)/diff(ribbonrange)
-               at <- bb.rib$yrange[1] + scal * (ribbonticks - ribbonrange[1])
+               if(trivial) {
+                 at <- mean(bb.rib$yrange)
+               } else {
+                 scal <- diff(bb.rib$yrange)/diff(ribbonrange)
+                 at <- bb.rib$yrange[1] + scal * (ribbonticks - ribbonrange[1])
+               }
                axisargs <- append(axisargs, list(at=at))
                posargs <- list(pos=bb.rib$xrange[1],
                                yaxp=c(bb.rib$yrange, length(ribbonticks)))
              },
              top={
-               scal <- diff(bb.rib$xrange)/diff(ribbonrange)
-               at <- bb.rib$xrange[1] + scal * (ribbonticks - ribbonrange[1])
+               if(trivial) {
+                 at <- mean(bb.rib$xrange)
+               } else {
+                 scal <- diff(bb.rib$xrange)/diff(ribbonrange)
+                 at <- bb.rib$xrange[1] + scal * (ribbonticks - ribbonrange[1])
+               }
                axisargs <- append(axisargs, list(at=at))
                posargs <- list(pos=bb.rib$yrange[2],
                                xaxp=c(bb.rib$xrange, length(ribbonticks)))
              },
              bottom={
-               scal <- diff(bb.rib$xrange)/diff(ribbonrange)
-               at <- bb.rib$xrange[1] + scal * (ribbonticks - ribbonrange[1])
+               if(trivial) {
+                 at <- mean(bb.rib$xrange)
+               } else {
+                 scal <- diff(bb.rib$xrange)/diff(ribbonrange)
+                 at <- bb.rib$xrange[1] + scal * (ribbonticks - ribbonrange[1])
+               }
                axisargs <- append(axisargs, list(at=at))
                posargs <- list(pos=bb.rib$yrange[1],
                                xaxp=c(bb.rib$xrange, length(ribbonticks)))
@@ -886,7 +908,7 @@ contour.im <- function (x, ..., main, axes=FALSE, add=FALSE,
                         col=par("fg"), 
                         clipwin=NULL, show.all=!add, do.plot=TRUE)
 {
-  defaultmain <- deparse(substitute(x))
+  defaultmain <- if(missing(main)) short.deparse(substitute(x)) else NULL
   dotargs <- list(...)
   bb <- Frame(x)
   xtype <- x$type
