@@ -1,7 +1,7 @@
 #
 #  psp.R
 #
-#  $Revision: 1.110 $ $Date: 2022/01/04 05:30:06 $
+#  $Revision: 1.116 $ $Date: 2024/06/16 02:06:24 $
 #
 # Class "psp" of planar line segment patterns
 #
@@ -128,7 +128,7 @@ as.psp.data.frame <- function(x, ..., window=NULL, marks=NULL,
     dy <- sin(x$angle) * rr
     bb <- boundingbox(window)
     rmax <- max(rr)
-    bigbox <- owin(bb$xrange + c(-1,1) * rmax, bb$yrange + c(-1,1) * rmax)
+    bigbox <- owinInternalRect(bb$xrange + c(-1,1) * rmax, bb$yrange + c(-1,1) * rmax)
     pattern <- psp(x$xmid - dx, x$ymid - dy, x$xmid + dx, x$ymid + dy,
                    window=bigbox,check=FALSE)
     out <- pattern[window]
@@ -188,7 +188,7 @@ as.psp.default <- function(x, ..., window=NULL, marks=NULL,
     window <- as.owin(window)
     bb <- boundingbox(window)
     rmax <- max(rr)
-    bigbox <- owin(bb$xrange + c(-1,1) * rmax, bb$yrange + c(-1,1) * rmax)
+    bigbox <- owinInternalRect(bb$xrange + c(-1,1) * rmax, bb$yrange + c(-1,1) * rmax)
     pattern <- psp(x$x - dx, x$y - dy, x$x + dx, x$y + dy,
                    window=bigbox, marks=marks, check=FALSE)
     clipped <- pattern[window]
@@ -578,27 +578,34 @@ is.empty.psp <- function(x) { return(x$n == 0) }
 
 identify.psp <- function(x, ..., labels=seq_len(nsegments(x)),
                          n=nsegments(x), plot=TRUE) {
+  if(dev.cur() == 1 && interactive()) {
+    eval(substitute(plot(X), list(X=substitute(x))))
+  }
   Y <- x
-  W <- as.owin(Y)
+  B <- Frame(Y)
+  Bplus <- grow.rectangle(B, max(sidelengths(B))/4)
   mids <- midpoints.psp(Y)
-  poz <- c(1, 2,4, 3)[(floor(angles.psp(Y)/(pi/4)) %% 4) + 1L]
+  poz <- c(1, 2, 4, 3)[(floor(angles.psp(Y)/(pi/4)) %% 4) + 1L]
+  gp <- if(plot) graphicsPars("lines") else NULL
   if(!(is.numeric(n) && (length(n) == 1) && (n %% 1 == 0) && (n >= 0)))
     stop("n should be a single integer")
   out <- integer(0)
   while(length(out) < n) {
-    xy <- spatstatLocator(1)
-    # check for interrupt exit
+    xy <- spatstatLocator(1, type="n")
+    ## check for interrupt exit
     if(length(xy$x) == 0)
       return(out)
-    # find nearest segment
-    X <- ppp(xy$x, xy$y, window=W)
+    ## find nearest segment
+    X <- ppp(xy$x, xy$y, window=Bplus)
     ident <- project2segment(X, Y)$mapXY
-    # add to list
-    if(ident %in% out) {
+    if(length(ident) == 0) {
+      cat("Query location is too far away\n")
+    } else if(ident %in% out) {
       cat(paste("Segment", ident, "already selected\n"))
     } else {
+      ## add to list
       if(plot) {
-        # Display
+        ## Display
         mi <- mids[ident]
         li <- labels[ident]
         po <- poz[ident]
@@ -607,15 +614,20 @@ identify.psp <- function(x, ..., labels=seq_len(nsegments(x)),
         dont.complain.about(li, mix, miy)
         do.call.matched(graphics::text.default,
                         resolve.defaults(list(x=quote(mix), 
-					      y=quote(miy), 
-				              labels=quote(li)),
+                                              y=quote(miy), 
+                                              labels=quote(li)),
                                          list(...),
                                          list(pos=po)))
+        do.call.matched(plot.psp,
+                        resolve.defaults(list(x=Y[ident], add=TRUE),
+                                         list(...),
+                                         list(col="blue", lwd=2)),
+                        extrargs=gp)
       }
       out <- c(out, ident)
     }
   }
-  # exit if max n reached
+  ## exit if max n reached
   return(out)
 }
 
@@ -678,7 +690,7 @@ intensity.psp <- function(X, ..., weights=NULL) {
   }
   ## weighted case 
   if(is.numeric(weights)) {
-    check.nvector(weights, nsegments(X), things="segments")
+    check.nvector(weights, nsegments(X), things="segments", vname="weights")
   } else if(is.expression(weights)) {
     # evaluate expression in data frame of coordinates and marks
     df <- as.data.frame(X)
@@ -686,7 +698,7 @@ intensity.psp <- function(X, ..., weights=NULL) {
     eval.weights <- try(eval(weights, envir=df, enclos=pf))
     if(inherits(eval.weights, "try-error"))
       stop("Unable to evaluate expression for weights", call.=FALSE)
-    if(!check.nvector(eval.weights, nsegments(X), fatal=FALSE, warn=TRUE))
+    if(!check.nvector(eval.weights, nsegments(X), fatal=FALSE, warn=TRUE, vname="eval(weights)"))
       stop("Result of evaluating the expression for weights has wrong format")
     weights <- eval.weights
   } else stop("Unrecognised format for argument 'weights'")
